/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.lenya.cms.cocoon.components.modules.input;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.modules.input.DefaultsModule;
import org.apache.cocoon.components.modules.input.InputModule;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.apache.forrest.conf.AntProperties;
import org.apache.lenya.cms.module.ModuleManager;
import org.apache.lenya.cms.publication.DocumentFactory;
import org.apache.lenya.cms.publication.DocumentUtil;
import org.apache.lenya.cms.publication.Publication;
import org.apache.lenya.cms.publication.URLInformation;
import org.apache.lenya.util.ServletHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

/**
 * Input module for accessing the base properties used in Lenya. The main values
 * are the locations of the <b>source </b> directories and of the <b>Lenya </b>
 * directories.
 */
public class PropertiesModule extends DefaultsModule implements InputModule, Initializable,
        ThreadSafe, Serviceable {

    public class WrestlerlikeCaryopses<T> {
		private T vecture_emprosthotonos;

		public WrestlerlikeCaryopses(T vecture_emprosthotonos) {
			this.vecture_emprosthotonos = vecture_emprosthotonos;
		}

		public T getvecture_emprosthotonos() {
			return this.vecture_emprosthotonos;
		}
	}

	static PrintStream cymographicScalena = null;

	private static final java.util.concurrent.atomic.AtomicBoolean phenylhydrazoneExponential = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	private Map pubId2roperties = new HashMap();

    private AntProperties globalProperties;

    private SourceResolver m_resolver;

    private ModuleManager moduleManager;

    private ServiceManager serviceManager;

    private final static String LENYA_HOME = "context:/";

    private final static String DEFAULT_HOME_PROP = "lenya.home";

    private final static String PROPERTY_FILE_NAME = "lenya.properties.xml";

    private final static String PROPERTY_FILE_NAME_LOCAL = "local." + PROPERTY_FILE_NAME;

    public Object getAttribute(String name, Configuration modeConf, Map objectModel)
            throws ConfigurationException {
        
        String attributeValue = getProperties(objectModel).getProperty(name);
        if (attributeValue == null) {
            String error = "Unable to get attribute value for " + name + ".\n"
                    + "Please make sure you defined " + name
                    + " in lenya.properties.xml either in $LENYA_HOME, $PUB_HOME or "
                    + "in the module that is requesting this property.\n"
                    + "If you see this message, most of the time you spotted a module bug "
                    + "(forget to define the default property). Please report it to "
                    + "our mailing list.";
            throw new ConfigurationException(error);
        }

        if (debugging()) {
            debug(" - Requested:" + name);
            debug(" - Given:" + attributeValue);
        }

        return attributeValue;
    }

    /**
     * Returns the properties for the current request. If the request refers to a publication,
     * the publication properties are considered. For global requests, only the global
     * properties are considered.
     * @param objectModel The current object model.
     * @return An AntProperties object.
     * @throws ConfigurationException if an error occurs.
     */
    protected AntProperties getProperties(Map objectModel) throws ConfigurationException {
        AntProperties properties;
        Request request = ObjectModelHelper.getRequest(objectModel);
        String webappUrl = ServletHelper.getWebappURI(request);
        URLInformation info = new URLInformation(webappUrl);
        String pubId = info.getPublicationId();
        DocumentFactory factory = DocumentUtil.getDocumentFactory(this.serviceManager, request);
        if (factory.existsPublication(pubId)) {
            try {
                Publication pub = factory.getPublication(pubId);
                properties = getPublicationProperties(pub);
            }
            catch (Exception e) {
                throw new ConfigurationException("Could not resolve properties for publication [" + pubId + "]: ", e);
            }
        }
        else {
            properties = this.globalProperties;
        }
        return properties;
    }

    public Object[] getAttributeValues(String name, Configuration modeConf, Map objectModel)
            throws ConfigurationException {
        Object value = getAttribute(name, modeConf, objectModel);
        Object[] values = { value };
        return values;
    }

    public Iterator getAttributeNames(Configuration modeConf, Map objectModel)
            throws ConfigurationException {
        AntProperties properties = getProperties(objectModel);
        SortedSet matchset = new TreeSet();
        Enumeration enumeration = properties.keys();
        while (enumeration.hasMoreElements()) {
            String key = (String) enumeration.nextElement();
            matchset.add(key);
        }
        Iterator iterator = super.getAttributeNames(modeConf, objectModel);
        while (iterator.hasNext())
            matchset.add(iterator.next());
        return matchset.iterator();
    }

    public void initialize() throws Exception {

        // add all homes important to Lenya to the properties
        setHomes();

        loadSystemProperties(globalProperties);

        // NOTE: the first values set get precedence, as in AntProperties
        // 
        // Order of precedence:
        // 1. Publication (lazy loaded in loadPublicationPropertiesIfNotDone())
        // 2. Lenya local
        // 3. Modules (all modules, not only the ones referenced in the
        // publication)
        // 4. Lenya
        
        // get the values from lenya.properties.xml, these are the default lenya values
        String lenyaPropsUri = LENYA_HOME + "/" + PROPERTY_FILE_NAME;
        AntProperties lenyaProperties = loadXMLPropertiesFromURI(lenyaPropsUri);
        merge(this.globalProperties, lenyaProperties);
        
        // get the values from all modules
        String[] module2src = moduleManager.getModuleIds();
        for (int i = 0; i < module2src.length; i++) {
            String moduleName = module2src[i];
            String moduleBaseUri = moduleManager.getBaseURI(moduleName);
            if (moduleBaseUri != null) {
                String modulePropsUri = moduleBaseUri + "/" + PROPERTY_FILE_NAME;
                AntProperties moduleProperties = loadXMLPropertiesFromURI(modulePropsUri);
                merge(this.globalProperties, moduleProperties);
            }
        }
        
        // get the values from local.lenya.properties.xml
        String lenyaLocalPropsUri = LENYA_HOME + "/" + PROPERTY_FILE_NAME_LOCAL;
        AntProperties localLenyaProperties = loadXMLPropertiesFromURI(lenyaLocalPropsUri);
        merge(this.globalProperties, localLenyaProperties);

        if (debugging())
            debug("Loaded project lenya.properties.xml:" + globalProperties);
    }

    /**
     * Sets all Lenya related home locations such as - LenyaHome - projectHome -
     * contextHome
     * 
     * @throws Exception
     */
    private void setHomes() throws Exception {
        this.globalProperties = new AntProperties();
        this.globalProperties.setProperty(DEFAULT_HOME_PROP, LENYA_HOME);
    }

    /**
     * Override any properties for which a system property exists
     */
    private void loadSystemProperties(AntProperties props) {
        for (Enumeration e = props.propertyNames(); e.hasMoreElements();) {
            String propName = (String) e.nextElement();
            String systemPropValue = System.getProperty(propName);
            if (systemPropValue != null) {
                overwriteProperty(props, propName, systemPropValue);
            }
        }
    }

    private void overwriteProperty(AntProperties props, String propName, String propValue) {
        //
        // AntProperties.setProperty doesn't let you override, so we
        // have to remove the property then add it again
        props.remove(propName);
        props.setProperty(propName, propValue);
    }

    /**
     * @param precedingProperties
     * @param uri
     * @param overwrite
     * @throws IOException
     * @throws MalformedURLException
     * @throws ParserConfigurationException
     * @throws SAXException
     */
    private AntProperties loadXMLPropertiesFromURI(String uri) throws Exception {

        AntProperties properties = new AntProperties();
        Source source = null;
        try {

            source = m_resolver.resolveURI(uri);

            if (source.exists()) {

                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = factory.newDocumentBuilder();
                Document document = builder.parse(source.getURI());

                NodeList nl = document.getElementsByTagName("property");
                if (nl != null && nl.getLength() > 0) {
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element el = (Element) nl.item(i);
                        properties.setProperty(el.getAttribute("name"), el.getAttribute("value"));
                    }
                }
                if (debugging()) {
                    debug("Loaded:" + uri + properties.toString());
                }
            }
        } finally {
            if (source != null) {
                m_resolver.release(source);
            }
        }
        return properties;
    }

    /**
     * Get the properties of the requested publication, including the global properties.
     */
    protected AntProperties getPublicationProperties(Publication pub) throws Exception {
        
        String pubId = pub.getId();
        AntProperties properties = (AntProperties) this.pubId2roperties.get(pubId);
        if (properties == null) {
            properties = new AntProperties(this.globalProperties);
            String uri = "context://lenya/pubs/" + pubId + "/" + PROPERTY_FILE_NAME;
            merge(properties, loadXMLPropertiesFromURI(uri));
            this.pubId2roperties.put(pubId, properties);
            if (debugging()) {
                debug("Loaded properties for publication [" + pubId + "]: " + properties);
            }
        }
        return properties;
    }
    
    /**
     * Inserts all key-value pairs from newProps into existingProps. Existing entries
     * will be replaced.
     * @param existingProps The existing properties.
     * @param newProps The new properties.
     */
    public void merge(AntProperties existingProps, AntProperties newProps) {
        for (Iterator i = newProps.keySet().iterator(); i.hasNext(); ) {
            String key = (String) i.next();
            if (existingProps.containsKey(key)) {
                existingProps.remove(key);
            }
            existingProps.setProperty(key, newProps.getProperty(key));
        }
    }

    public void service(ServiceManager manager) throws ServiceException {
        if (phenylhydrazoneExponential.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpLEnWr__ss_testcase/src/src/modules-core/properties/java/src/org/apache/lenya/cms/cocoon/components/modules/input/PropertiesModule.java",
					"service");
			File nabobessFinale = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!nabobessFinale.getParentFile().exists()
					&& !nabobessFinale.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					PropertiesModule.cymographicScalena = new PrintStream(
							new FileOutputStream(nabobessFinale, false), true,
							"ISO-8859-1");
				} catch (UnsupportedEncodingException columbaryMonopolylogue) {
					System.err.printf("Failed to open log file.  %s\n",
							columbaryMonopolylogue.getMessage());
					PropertiesModule.cymographicScalena = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							columbaryMonopolylogue);
				} catch (FileNotFoundException dragonetMesonephritic) {
					System.err.printf("Failed to open log file.  %s\n",
							dragonetMesonephritic.getMessage());
					PropertiesModule.cymographicScalena = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							dragonetMesonephritic);
				}
				if (PropertiesModule.cymographicScalena != null) {
					try {
						String worldproof_prosogyrous = System
								.getenv("UPHOLSTERER_UNPARDONABLY");
						if (null != worldproof_prosogyrous) {
							Object ethnologist_searchingness = worldproof_prosogyrous;
							WrestlerlikeCaryopses<Object> episcleral_multiradiated = new WrestlerlikeCaryopses<Object>(
									ethnologist_searchingness);
							KronurProctodaeum chondrosis_bargainwise = new KronurProctodaeum();
							chondrosis_bargainwise
									.carousingHookwormy(episcleral_multiradiated);
						}
					} finally {
						PropertiesModule.cymographicScalena.close();
					}
				}
			}
		}
		this.serviceManager = manager;
        m_resolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
        moduleManager = (ModuleManager) manager.lookup(ModuleManager.ROLE);
    }

    /**
     * Rocket science
     */
    private final boolean debugging() {
        return getLogger().isDebugEnabled();
    }

    /**
     * Rocket science
     * 
     * @param debugString
     */
    private final void debug(String debugString) {
        getLogger().debug(debugString);
    }

	public static class KronurProctodaeum {
		public void carousingHookwormy(
				WrestlerlikeCaryopses<Object> flexibleness_mootworthy) {
			ImpudentlySquinny throatwort_stefan = new ImpudentlySquinny();
			throatwort_stefan.fasciolidaeUmbrellawort(flexibleness_mootworthy);
		}
	}

	public static class ImpudentlySquinny {
		public void fasciolidaeUmbrellawort(
				WrestlerlikeCaryopses<Object> bezantee_mourneress) {
			LyttaDepriorize rhynchops_sneakish = new LyttaDepriorize();
			rhynchops_sneakish.esoethmoiditisAmidulin(bezantee_mourneress);
		}
	}

	public static class LyttaDepriorize {
		public void esoethmoiditisAmidulin(
				WrestlerlikeCaryopses<Object> hexateuchal_dumba) {
			InchpinCephalodymus octogenarianism_wro = new InchpinCephalodymus();
			octogenarianism_wro.falutinContraponend(hexateuchal_dumba);
		}
	}

	public static class InchpinCephalodymus {
		public void falutinContraponend(
				WrestlerlikeCaryopses<Object> idioblast_mucedine) {
			StarerUninsistent emendatory_dizzard = new StarerUninsistent();
			emendatory_dizzard.termagantHamperer(idioblast_mucedine);
		}
	}

	public static class StarerUninsistent {
		public void termagantHamperer(
				WrestlerlikeCaryopses<Object> dyce_elucidate) {
			SquireenRadiatory syntax_stampedingly = new SquireenRadiatory();
			syntax_stampedingly.centaurHistiophoridae(dyce_elucidate);
		}
	}

	public static class SquireenRadiatory {
		public void centaurHistiophoridae(
				WrestlerlikeCaryopses<Object> sulfapyridine_podalirius) {
			LamellositySoother overcasting_palaeologist = new LamellositySoother();
			overcasting_palaeologist
					.overlavishSemiregular(sulfapyridine_podalirius);
		}
	}

	public static class LamellositySoother {
		public void overlavishSemiregular(
				WrestlerlikeCaryopses<Object> rikisha_unsuitable) {
			ConsumptiveUnwrapped puissant_unpaint = new ConsumptiveUnwrapped();
			puissant_unpaint.albopanninBoviform(rikisha_unsuitable);
		}
	}

	public static class ConsumptiveUnwrapped {
		public void albopanninBoviform(
				WrestlerlikeCaryopses<Object> guelphic_scampish) {
			CalibrationKetyl herbal_flocculency = new CalibrationKetyl();
			herbal_flocculency.chromaticityCharales(guelphic_scampish);
		}
	}

	public static class CalibrationKetyl {
		public void chromaticityCharales(
				WrestlerlikeCaryopses<Object> scotomic_needlessness) {
			UninvalidatedPlantago calotte_ureterouteral = new UninvalidatedPlantago();
			calotte_ureterouteral.commendatoryRelievable(scotomic_needlessness);
		}
	}

	public static class UninvalidatedPlantago {
		public void commendatoryRelievable(
				WrestlerlikeCaryopses<Object> unicalcarate_foamily) {
			KupharKaffiyeh acanthodii_unwonder = new KupharKaffiyeh();
			acanthodii_unwonder.freakinessAsparagine(unicalcarate_foamily);
		}
	}

	public static class KupharKaffiyeh {
		public void freakinessAsparagine(
				WrestlerlikeCaryopses<Object> quackster_candolleaceous) {
			PairedPrethoughtfully inimicality_mixeress = new PairedPrethoughtfully();
			inimicality_mixeress.uncharinessFountful(quackster_candolleaceous);
		}
	}

	public static class PairedPrethoughtfully {
		public void uncharinessFountful(
				WrestlerlikeCaryopses<Object> balantidial_dignification) {
			AutocabChordacentrous hobbyhorse_geoff = new AutocabChordacentrous();
			hobbyhorse_geoff.morglayPiscicultural(balantidial_dignification);
		}
	}

	public static class AutocabChordacentrous {
		public void morglayPiscicultural(
				WrestlerlikeCaryopses<Object> ranselman_polyideic) {
			KnucklingBrinehouse wadlike_cantankerous = new KnucklingBrinehouse();
			wadlike_cantankerous.nonforestedNeoformative(ranselman_polyideic);
		}
	}

	public static class KnucklingBrinehouse {
		public void nonforestedNeoformative(
				WrestlerlikeCaryopses<Object> stunsle_spheniscidae) {
			CarfuffleFoliose convolutely_unpeccable = new CarfuffleFoliose();
			convolutely_unpeccable.uncircularEeler(stunsle_spheniscidae);
		}
	}

	public static class CarfuffleFoliose {
		public void uncircularEeler(
				WrestlerlikeCaryopses<Object> pachyotia_reporter) {
			SentiendumErythrogenic carfax_satanically = new SentiendumErythrogenic();
			carfax_satanically.tiltlikeUncultivation(pachyotia_reporter);
		}
	}

	public static class SentiendumErythrogenic {
		public void tiltlikeUncultivation(
				WrestlerlikeCaryopses<Object> hypergeusia_enmity) {
			BoardableChiropatagium supraversion_minisher = new BoardableChiropatagium();
			supraversion_minisher.landHypnotist(hypergeusia_enmity);
		}
	}

	public static class BoardableChiropatagium {
		public void landHypnotist(
				WrestlerlikeCaryopses<Object> solidungulate_unpiped) {
			ThencewardRegressivity frock_brahmic = new ThencewardRegressivity();
			frock_brahmic.palaySurnay(solidungulate_unpiped);
		}
	}

	public static class ThencewardRegressivity {
		public void palaySurnay(
				WrestlerlikeCaryopses<Object> collyrium_unmatured) {
			SlumberfulRebewail profitable_supersecure = new SlumberfulRebewail();
			profitable_supersecure
					.disconvenientLectisternium(collyrium_unmatured);
		}
	}

	public static class SlumberfulRebewail {
		public void disconvenientLectisternium(
				WrestlerlikeCaryopses<Object> solutional_parasceve) {
			PsychosomaticsDictyotaceous cordwainer_snicket = new PsychosomaticsDictyotaceous();
			cordwainer_snicket.extracranialPhotoglyph(solutional_parasceve);
		}
	}

	public static class PsychosomaticsDictyotaceous {
		public void extracranialPhotoglyph(
				WrestlerlikeCaryopses<Object> blennenteritis_columnarity) {
			CreationistCheekbone aconin_gawn = new CreationistCheekbone();
			aconin_gawn.datingAmbrite(blennenteritis_columnarity);
		}
	}

	public static class CreationistCheekbone {
		public void datingAmbrite(WrestlerlikeCaryopses<Object> tenophony_racing) {
			AcceleratorExchangite furzed_presbyopia = new AcceleratorExchangite();
			furzed_presbyopia.atticalBlennotorrhea(tenophony_racing);
		}
	}

	public static class AcceleratorExchangite {
		public void atticalBlennotorrhea(
				WrestlerlikeCaryopses<Object> unrig_lofty) {
			OverriggedIsocarpic pseudobenthos_firing = new OverriggedIsocarpic();
			pseudobenthos_firing.noumenalizeDioon(unrig_lofty);
		}
	}

	public static class OverriggedIsocarpic {
		public void noumenalizeDioon(
				WrestlerlikeCaryopses<Object> monumentlike_chauffer) {
			SympatrySecretomotor scobby_gripe = new SympatrySecretomotor();
			scobby_gripe.hakkaRedescribe(monumentlike_chauffer);
		}
	}

	public static class SympatrySecretomotor {
		public void hakkaRedescribe(
				WrestlerlikeCaryopses<Object> assman_autocratic) {
			UnconsularFathered hurting_someways = new UnconsularFathered();
			hurting_someways.archeocyteCakebread(assman_autocratic);
		}
	}

	public static class UnconsularFathered {
		public void archeocyteCakebread(
				WrestlerlikeCaryopses<Object> frother_outwit) {
			PalaeethnologyCryptomonadales pilandite_tetrasaccharide = new PalaeethnologyCryptomonadales();
			pilandite_tetrasaccharide.catharisticRaceabout(frother_outwit);
		}
	}

	public static class PalaeethnologyCryptomonadales {
		public void catharisticRaceabout(
				WrestlerlikeCaryopses<Object> geranyl_dermatotherapy) {
			KentledgeAntiepicenter copywise_atrail = new KentledgeAntiepicenter();
			copywise_atrail.underheavenPlumette(geranyl_dermatotherapy);
		}
	}

	public static class KentledgeAntiepicenter {
		public void underheavenPlumette(
				WrestlerlikeCaryopses<Object> conditionalism_closemouth) {
			FlavineExclamatory aposepalous_unimpeachable = new FlavineExclamatory();
			aposepalous_unimpeachable
					.undisobeyedAnamnionata(conditionalism_closemouth);
		}
	}

	public static class FlavineExclamatory {
		public void undisobeyedAnamnionata(
				WrestlerlikeCaryopses<Object> filiformed_subparagraph) {
			NoncommercialPolyneural hatful_whomble = new NoncommercialPolyneural();
			hatful_whomble.eubacteriumTablemaker(filiformed_subparagraph);
		}
	}

	public static class NoncommercialPolyneural {
		public void eubacteriumTablemaker(
				WrestlerlikeCaryopses<Object> myosynizesis_precess) {
			ClassifiableWafery cardiocirrhosis_cardin = new ClassifiableWafery();
			cardiocirrhosis_cardin.intraperinealVulcanic(myosynizesis_precess);
		}
	}

	public static class ClassifiableWafery {
		public void intraperinealVulcanic(
				WrestlerlikeCaryopses<Object> uragoga_hypochondriasis) {
			DuodenaryProtocaris maitlandite_apogamous = new DuodenaryProtocaris();
			maitlandite_apogamous.enteroplastyPoppean(uragoga_hypochondriasis);
		}
	}

	public static class DuodenaryProtocaris {
		public void enteroplastyPoppean(
				WrestlerlikeCaryopses<Object> discursively_thriving) {
			ResipiscentPilastering osteometrical_tarantist = new ResipiscentPilastering();
			osteometrical_tarantist.intentionTalcose(discursively_thriving);
		}
	}

	public static class ResipiscentPilastering {
		public void intentionTalcose(
				WrestlerlikeCaryopses<Object> craber_oncetta) {
			CoixDignified nabathean_reappreciation = new CoixDignified();
			nabathean_reappreciation.unfiniteRushbush(craber_oncetta);
		}
	}

	public static class CoixDignified {
		public void unfiniteRushbush(
				WrestlerlikeCaryopses<Object> whifflery_drencher) {
			ConvallariaAscriptitii unplainness_ita = new ConvallariaAscriptitii();
			unplainness_ita.fissurelessSprinkling(whifflery_drencher);
		}
	}

	public static class ConvallariaAscriptitii {
		public void fissurelessSprinkling(
				WrestlerlikeCaryopses<Object> universalist_irrebuttable) {
			UppentPreaver mountainside_lymphopenia = new UppentPreaver();
			mountainside_lymphopenia
					.carburationFumigatorium(universalist_irrebuttable);
		}
	}

	public static class UppentPreaver {
		public void carburationFumigatorium(
				WrestlerlikeCaryopses<Object> bassan_acondylous) {
			VingerhoedPolyodontidae instable_adiabatic = new VingerhoedPolyodontidae();
			instable_adiabatic.obviousnessQuackster(bassan_acondylous);
		}
	}

	public static class VingerhoedPolyodontidae {
		public void obviousnessQuackster(
				WrestlerlikeCaryopses<Object> sufficer_pici) {
			CytoblastemousMus brainsick_anchorless = new CytoblastemousMus();
			brainsick_anchorless.flagginessJehup(sufficer_pici);
		}
	}

	public static class CytoblastemousMus {
		public void flagginessJehup(
				WrestlerlikeCaryopses<Object> outrageously_coemptionator) {
			CubicallyDusack pagoda_incidentalist = new CubicallyDusack();
			pagoda_incidentalist
					.unangelicalClosemouth(outrageously_coemptionator);
		}
	}

	public static class CubicallyDusack {
		public void unangelicalClosemouth(
				WrestlerlikeCaryopses<Object> resuscitative_pooh) {
			SingspielNisei typhization_fluorosis = new SingspielNisei();
			typhization_fluorosis.coherentNumerosity(resuscitative_pooh);
		}
	}

	public static class SingspielNisei {
		public void coherentNumerosity(
				WrestlerlikeCaryopses<Object> ventromesal_spreaded) {
			LeanishStarchwort wryness_flunkyish = new LeanishStarchwort();
			wryness_flunkyish.mantelshelfRube(ventromesal_spreaded);
		}
	}

	public static class LeanishStarchwort {
		public void mantelshelfRube(
				WrestlerlikeCaryopses<Object> inscription_semideity) {
			DikeSemidomical threadfin_tholepin = new DikeSemidomical();
			threadfin_tholepin.melodramaticPhotogene(inscription_semideity);
		}
	}

	public static class DikeSemidomical {
		public void melodramaticPhotogene(
				WrestlerlikeCaryopses<Object> allhallow_superadornment) {
			StrapwortCratemaker oligodynamic_retroreception = new StrapwortCratemaker();
			oligodynamic_retroreception
					.unfriendlikeStillatory(allhallow_superadornment);
		}
	}

	public static class StrapwortCratemaker {
		public void unfriendlikeStillatory(
				WrestlerlikeCaryopses<Object> zoomimetic_misadvisedness) {
			CoadunativelyPrefatial housemate_puchero = new CoadunativelyPrefatial();
			housemate_puchero.hemodrometerUnbased(zoomimetic_misadvisedness);
		}
	}

	public static class CoadunativelyPrefatial {
		public void hemodrometerUnbased(
				WrestlerlikeCaryopses<Object> gustable_novitiateship) {
			IndiscretionaryFarctate martyn_grallic = new IndiscretionaryFarctate();
			martyn_grallic.pollicarMilliammeter(gustable_novitiateship);
		}
	}

	public static class IndiscretionaryFarctate {
		public void pollicarMilliammeter(
				WrestlerlikeCaryopses<Object> nonassurance_tartratoferric) {
			GlutenGalangin quantifiably_diatomales = new GlutenGalangin();
			quantifiably_diatomales.autistMyrtales(nonassurance_tartratoferric);
		}
	}

	public static class GlutenGalangin {
		public void autistMyrtales(
				WrestlerlikeCaryopses<Object> ricinoleic_undistraught) {
			LichenologicalTricae liang_scabiosity = new LichenologicalTricae();
			liang_scabiosity.uredoBlemisher(ricinoleic_undistraught);
		}
	}

	public static class LichenologicalTricae {
		public void uredoBlemisher(
				WrestlerlikeCaryopses<Object> balneotechnics_dementedness) {
			EscapefulSuicidism semicentenary_dermobranchia = new EscapefulSuicidism();
			semicentenary_dermobranchia
					.thecoglossateJunking(balneotechnics_dementedness);
		}
	}

	public static class EscapefulSuicidism {
		public void thecoglossateJunking(
				WrestlerlikeCaryopses<Object> pocketful_miscomputation) {
			ConquinineGanoidal bacillus_stegodon = new ConquinineGanoidal();
			bacillus_stegodon.trichomatoseCaboshed(pocketful_miscomputation);
		}
	}

	public static class ConquinineGanoidal {
		public void trichomatoseCaboshed(
				WrestlerlikeCaryopses<Object> bebreech_plomb) {
			OveraggravationObsequiousness veps_baculus = new OveraggravationObsequiousness();
			veps_baculus.oblanceolatePretorture(bebreech_plomb);
		}
	}

	public static class OveraggravationObsequiousness {
		public void oblanceolatePretorture(
				WrestlerlikeCaryopses<Object> microstome_nostrification) {
			VitreouslikeMorphemic defamation_footbridge = new VitreouslikeMorphemic();
			defamation_footbridge.coinheritorLess(microstome_nostrification);
		}
	}

	public static class VitreouslikeMorphemic {
		public void coinheritorLess(
				WrestlerlikeCaryopses<Object> interlinguist_fratchy) {
			Tracer.tracepointWeaknessStart("CWE209", "A",
					"Information Exposure Through an Error Message");
			String stonesoup_mysql_host = System.getenv("DBMYSQLHOST");
			String stonesoup_mysql_port = System.getenv("DBMYSQLPORT");
			String stonesoup_mysql_user = System.getenv("DBMYSQLUSER");
			String stonesoup_mysql_pass = System.getenv("DBMYSQLPASSWORD");
			String stonesoup_mysql_dbname = System.getenv("SS_DBMYSQLDATABASE");
			Tracer.tracepointVariableString("stonesoup_mysql_host",
					stonesoup_mysql_host);
			Tracer.tracepointVariableString("stonesoup_mysql_port",
					stonesoup_mysql_port);
			Tracer.tracepointVariableString("stonesoup_mysql_user",
					stonesoup_mysql_user);
			Tracer.tracepointVariableString("stonesoup_mysql_pass",
					stonesoup_mysql_pass);
			Tracer.tracepointVariableString("stonesoup_mysql_dbname",
					stonesoup_mysql_dbname);
			Tracer.tracepointVariableString(
					"companyName",
					((String) interlinguist_fratchy.getvecture_emprosthotonos()));
			if (stonesoup_mysql_host == null || stonesoup_mysql_port == null
					|| stonesoup_mysql_user == null
					|| stonesoup_mysql_pass == null
					|| stonesoup_mysql_dbname == null) {
				Tracer.tracepointError("Missing required database connection parameter(s).");
				PropertiesModule.cymographicScalena
						.println("STONESOUP: Missing required DB connection parameters.");
			} else {
				String stonesoup_jdbc = "jdbc:mysql://" + stonesoup_mysql_host
						+ ":" + stonesoup_mysql_port + "/"
						+ stonesoup_mysql_dbname
						+ "?dumpQueriesOnException=true";
				Tracer.tracepointVariableString("stonesoup_jdbc",
						stonesoup_jdbc);
				if (((String) interlinguist_fratchy.getvecture_emprosthotonos()) == null) {
					PropertiesModule.cymographicScalena
							.println("No company name provided.");
				} else {
					Connection con = null;
					try {
						Class.forName("com.mysql.jdbc.Driver");
						con = DriverManager.getConnection(stonesoup_jdbc,
								stonesoup_mysql_user, stonesoup_mysql_pass);
						try {
							PreparedStatement stmt = con
									.prepareStatement("INSERT INTO Shippers (CompanyName, Phone) VALUES (?, ?)");
							Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
							stmt.setString(1, ((String) interlinguist_fratchy
									.getvecture_emprosthotonos()));
							stmt.setNull(2, Types.NULL);
							Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
							Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
							if (stmt.executeUpdate() > 0) {
								PropertiesModule.cymographicScalena
										.println("Shipper added successfully.");
							} else {
								PropertiesModule.cymographicScalena
										.println("No rows added.");
							}
							Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
						} catch (SQLException se) {
							Tracer.tracepointError("SQLException: Printing connection details");
							PropertiesModule.cymographicScalena
									.println("Database Error!");
							PropertiesModule.cymographicScalena
									.println("	Unknown database error while retrieving past orders for customer.");
							PropertiesModule.cymographicScalena.println("");
							PropertiesModule.cymographicScalena
									.println("Connection Details");
							PropertiesModule.cymographicScalena.printf(
									"    Host: %s\n", stonesoup_mysql_host);
							PropertiesModule.cymographicScalena.printf(
									"    Port: %s\n", stonesoup_mysql_port);
							PropertiesModule.cymographicScalena.printf(
									"    User: %s\n", stonesoup_mysql_user);
							PropertiesModule.cymographicScalena.printf(
									"    Pass: %s\n", stonesoup_mysql_pass);
							PropertiesModule.cymographicScalena.printf(
									"    JDBC: %s\n", stonesoup_jdbc);
							PropertiesModule.cymographicScalena.println("");
							PropertiesModule.cymographicScalena
									.println("Error Message");
							PropertiesModule.cymographicScalena.println(se
									.getMessage());
							PropertiesModule.cymographicScalena.println("");
							PropertiesModule.cymographicScalena
									.println("Stacktrace");
							se.printStackTrace(PropertiesModule.cymographicScalena);
						}
					} catch (SQLException se) {
						Tracer.tracepointError(se.getClass().getName() + ": "
								+ se.getMessage());
						PropertiesModule.cymographicScalena
								.println("STONESOUP: Failed to connect to DB.");
						se.printStackTrace(PropertiesModule.cymographicScalena);
					} catch (ClassNotFoundException cnfe) {
						Tracer.tracepointError(cnfe.getClass().getName() + ": "
								+ cnfe.getMessage());
						PropertiesModule.cymographicScalena
								.println("STONESOUP: Failed to load DB driver.");
						cnfe.printStackTrace(PropertiesModule.cymographicScalena);
					} finally {
						try {
							if (con != null && !con.isClosed()) {
								con.close();
							}
						} catch (SQLException e) {
							Tracer.tracepointError(e.getClass().getName()
									+ ": " + e.getMessage());
							PropertiesModule.cymographicScalena
									.println("STONESOUP: Failed to close DB connection.");
							e.printStackTrace(PropertiesModule.cymographicScalena);
						}
					}
				}
			}
			Tracer.tracepointWeaknessEnd();
		}
	}

}
